# -*- coding: utf-8 -*-
#!/bin/env python3

# Copyright (C) 2018 Gaby Launay

# Author: Gaby Launay  <gaby.launay@tutanota.com>
# URL: https://github.com/gabylaunay/pyDSA_core

# This file is part of pyDSA_core

# pyDSA_core is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

import numpy as np
import matplotlib.pyplot as plt
from IMTreatment.utils import ProgressCounter
from IMTreatment import Profile, plotlib as pplt
from . import helpers as hlp


"""  """

__author__ = "Gaby Launay"
__copyright__ = "Gaby Launay 2017"
__credits__ = ""
__email__ = "gaby.launay@tutanota.com"
__status__ = "Development"


class TemporalFits(object):
    def __init__(self, fits, temporaledges):
        """

        """
        self.baseline = temporaledges.baseline
        self.fits = fits
        self.times = temporaledges.times
        self.unit_times = temporaledges.unit_times
        self.dt = temporaledges.dt
        self.unit_x = temporaledges.unit_x
        self.unit_y = temporaledges.unit_y

    def __iter__(self):
        for i in range(len(self.fits)):
            yield self.fits[i]

    def __len__(self):
        return len(self.fits)

    def __getitem__(self, i):
        return self.fits[i]

    def compute_contact_angle(self, iteration_hook=None, verbose=False):
        """
        Compute the drop contact angles.
        """
        if verbose:
            pg = ProgressCounter(init_mess="Getting contact angles",
                                 nmb_max=len(self.fits),
                                 name_things='images',
                                 perc_interv=5)
        for i, fit in enumerate(self.fits):
            try:
                fit.compute_contact_angle()
            except Exception:
                pass
            if verbose:
                pg.print_progress()
            if iteration_hook is not None:
                iteration_hook(i, len(self.fits))

    def smooth_triple_points(self, tos='gaussian', size=None):
        """
        Smooth the position of the triple point.

        Parameters
        ==========
        tos : string, optional
            Type of smoothing, can be 'uniform' (default) or 'gaussian'
            (See ndimage module documentation for more details)
        size : number, optional
            Size of the smoothing (is radius for 'uniform' and
            sigma for 'gaussian').
            Default is 3 for 'uniform' and 1 for 'gaussian'.
        """
        # Get triple points as profiles
        x1 = []
        y1 = []
        x2 = []
        y2 = []
        t = []
        mask = []
        for i, fit in enumerate(self.fits):
            t.append(self.times[i])
            if fit.triple_pts is not None:
                x1.append(fit.triple_pts[0][0])
                x2.append(fit.triple_pts[1][0])
                y1.append(fit.triple_pts[0][1])
                y2.append(fit.triple_pts[1][1])
                mask.append(False)
            else:
                x1.append(np.nan)
                x2.append(np.nan)
                y1.append(np.nan)
                y2.append(np.nan)
                mask.append(True)
        # Smooth the profiles
        if not np.all(np.isnan(x1)):
            x1 = Profile(t, x1).smooth(tos='gaussian', size=size).y
            x2 = Profile(t, x2).smooth(tos='gaussian', size=size).y
            y1 = Profile(t, y1).smooth(tos='gaussian', size=size).y
            y2 = Profile(t, y2).smooth(tos='gaussian', size=size).y
            # Replace by the smoothed version
            for i, fit in enumerate(self.fits):
                if not mask[i]:
                    fit.triple_pts = [[x1[i], y1[i]], [x2[i], y2[i]]]

    def smooth_contact_angle(self, tos='gaussian', size=None):
        """
        Smooth the contact angles.

        Parameters
        ==========
        tos : string, optional
            Type of smoothing, can be 'uniform' (default) or 'gaussian'
            (See ndimage module documentation for more details)
        size : number, optional
            Size of the smoothing (is radius for 'uniform' and
            sigma for 'gaussian').
            Default is 3 for 'uniform' and 1 for 'gaussian'.
        """
        # Get the contact angles as profiles
        theta1 = []
        theta2 = []
        theta3 = []
        theta4 = []
        t = []
        mask = []
        mask2 = []
        for i, fit in enumerate(self.fits):
            t.append(self.times[i])
            if fit.thetas is not None:
                theta1.append(fit.thetas[0])
                theta2.append(fit.thetas[1])
                mask.append(False)
            else:
                theta1.append(np.nan)
                theta2.append(np.nan)
                mask.append(True)
            if fit.thetas_triple is not None:
                theta3.append(fit.thetas_triple[0])
                theta4.append(fit.thetas_triple[1])
                mask2.append(False)
            else:
                theta3.append(np.nan)
                theta4.append(np.nan)
                mask2.append(True)
        # smooth
        if not np.all(mask):
            theta1 = Profile(t, theta1).smooth(tos='gaussian', size=size).y
            theta2 = Profile(t, theta2).smooth(tos='gaussian', size=size).y
            for i, fit in enumerate(self.fits):
                if not mask[i]:
                    fit.thetas = [theta1[i], theta2[i]]
        if not np.all(mask2):
            theta3 = Profile(t, theta3).smooth(tos='gaussian', size=size).y
            theta4 = Profile(t, theta4).smooth(tos='gaussian', size=size).y
            for i, fit in enumerate(self.fits):
                if not mask2[i]:
                    fit.thetas_triple = [theta3[i], theta4[i]]

    def get_contact_angles(self):
        """
        Return the drop contact angles.
        """
        thetas = []
        for fit in self.fits:
            if fit.thetas is not None:
                thetas.append(fit.thetas)
            else:
                thetas.append([np.nan, np.nan])
        thetas = np.asarray(thetas)
        return thetas

    def get_triple_pts_contact_angles(self):
        """
        Return the contact angles at the triple point.
        """
        thetas = []
        for fit in self.fits:
            if fit.thetas_triple is not None:
                thetas.append(fit.thetas_triple)
            else:
                thetas.append([np.nan, np.nan])
        thetas = np.asarray(thetas)
        return thetas

    def get_base_diameters(self):
        """ Return the drop base diameter. """
        bds = []
        for fld in self.fits:
            try:
                bd = fld.get_base_diameter()
                bds.append(bd)
            except:
                bds.append(np.nan)
        return np.array(bds)

    def get_drop_centers(self):
        """ Return the drop center position. """
        bds = []
        for fld in self.fits:
            try:
                bd = fld.get_drop_center()
                bds.append(bd)
            except:
                bds.append([np.nan, np.nan])
        return np.array(bds)

    def get_drop_positions(self):
        """ Return the position of the droplet edges. """
        pt1s = []
        pt2s = []
        for fit in self.fits:
            pt1, pt2 = fit.get_drop_position()
            pt1s.append(pt1)
            pt2s.append(pt2)
        return np.array(pt1s), np.array(pt2s)

    def get_drop_heights(self):
        """ Return the height of the droplet center. """
        bds = []
        for fld in self.fits:
            try:
                bd = fld.get_drop_height()
                bds.append(bd)
            except:
                bds.append(np.nan)
        return np.array(bds)

    def get_drop_areas(self):
        """ Return the area of the 2D projection of the droplet. """
        bds = []
        for fld in self.fits:
            try:
                bd = fld.get_drop_area()
                bds.append(bd)
            except:
                bds.append(np.nan)
        return np.array(bds)

    def get_drop_volumes(self):
        """ Return the estiated volume of the droplet. """
        bds = []
        for fld in self.fits:
            try:
                bd = fld.get_drop_volume()
                bds.append(bd)
            except:
                bds.append(np.nan)
        return np.array(bds)

    def get_triple_points(self):
        """
        Return the drop triple points.
        """
        triple_pts1 = []
        triple_pts2 = []
        for fit in self.fits:
            if fit.triple_pts is not None:
                triple_pts1.append(fit.triple_pts[0])
                triple_pts2.append(fit.triple_pts[1])
            else:
                triple_pts1.append([np.nan, np.nan])
                triple_pts2.append([np.nan, np.nan])
        triple_pts1 = np.asarray(triple_pts1)
        triple_pts2 = np.asarray(triple_pts2)
        return triple_pts1, triple_pts2

    def get_ridge_height(self):
        """
        Return the ridge heights.
        """
        height1 = []
        height2 = []
        for fit in self.fits:
            h1, h2 = fit.get_ridge_height()
            height1.append(h1)
            height2.append(h2)
        return (np.array(height1, dtype=float),
                np.array(height2, dtype=float))

    def _get_inters_base_fit(self):
        inter = []
        for fit in self.fits:
            inter.append(fit._get_inters_base_fit())
        return np.asarray(inter)

    def display(self, displ_tp=True, displ_ca=True):
        displs = []
        length = len(self.fits)
        # Display triple points
        if displ_tp:
            xs = []
            ys = []
            for fit in self.fits:
                if fit is None:
                    xs.append([np.nan, np.nan])
                    ys.append([np.nan, np.nan])
                elif fit.triple_pts is None:
                    xs.append([np.nan, np.nan])
                    ys.append([np.nan, np.nan])
                else:
                    xs.append([fit.triple_pts[0][0],
                               fit.triple_pts[1][0]])
                    ys.append([fit.triple_pts[0][1],
                               fit.triple_pts[1][1]])
            if len(xs) != length or len(ys) != length:
                raise Exception()
            if not np.all(np.isnan(xs)):
                db = pplt.Displayer(xs, ys, ls='none', marker='o',
                                    kind='plot',
                                    color=self[0].colors[2])
                displs.append(db)
        # Display contact angles
        if np.any([fit.thetas
                   for fit in self.fits
                   if fit is not None] is not None) \
           and displ_ca:
            lines = [fit._get_angle_display_lines()
                     for fit in self.fits
                     if fit is not None]
            lines1 = []
            lines2 = []
            lines3 = []
            lines4 = []
            for line in lines:
                lines1.append(line[0])
                lines2.append(line[1])
                if len(line) > 2:
                    lines3.append(line[2])
                    lines4.append(line[3])
                else:
                    lines3.append([[np.nan, np.nan], [np.nan, np.nan]])
                    lines4.append([[np.nan, np.nan], [np.nan, np.nan]])
            for line in [lines1, lines2, lines3, lines4]:
                line = np.array(line)
                if not np.all(np.isnan(line)):
                    if len(line[:, 0]) != length or len(line[:, 1]) != length:
                        raise Exception()
                    db = pplt.Displayer(line[:, 0], line[:, 1],
                                        kind='plot', color=self[0].colors[0])
                    displs.append(db)
        # Add button manager
        if len(displs) != 0:
            bm = pplt.ButtonManager(displs)
            return bm

    def display_summary(self, figsize=None):
        """
        Display a summary of the drop parameters evolution.
        """
        bdp = self._get_inters_base_fit()
        radii = self.get_base_diameters()
        thetas = self.get_contact_angles()
        # triple_pts1, triple_pts2 = self.get_triple_points()
        ts = np.arange(0, len(self.fits)*self.dt, self.dt)[0: len(bdp)]
        fig, axs = plt.subplots(2, 1, figsize=figsize)
        # Drop dimensions
        plt.sca(axs[0])
        plt.plot(bdp[:, 0, 0], ts, label="Contact (left)")
        plt.plot(bdp[:, 1, 0], ts, label="Contact (right)")
        plt.plot(radii, ts, label="Base radius")
        plt.xlabel('x {}'.format(self.unit_x.strUnit()))
        plt.ylabel('Time {}'.format(self.unit_times.strUnit()))
        plt.legend(loc=0)
        # Contact angles
        plt.sca(axs[1])
        plt.plot(-thetas[:, 0], ts, label="Angle (left)")
        plt.plot(180 - thetas[:, 1], ts, label="Angle (right)")
        plt.ylabel('Time {}'.format(self.unit_times.strUnit()))
        plt.xlabel('[Deg]')
        plt.legend(loc=0)


class TemporalSplineFits(TemporalFits):
    def detect_triple_points(self, smooth=None, use_x_minima=False,
                             verbose=False):
        """
        Compute the triple points (water, oil and air interfaces) positions.

        Parameters
        ==========
        smooth: number
           Smoothing factor for the triple point position.
        use_x_minima: boolean
            If True, try to define the triple point as the minimal x values and
            fall back to the curvature method if necessary.
            (default to False).

        Returns
        =======
        tripl_pts: 2x2 array of numbers
           Position of the triple points for each edge ([pt1, pt2])
        """
        if verbose:
            pg = ProgressCounter(init_mess="Detecting triple point positions",
                                 nmb_max=len(self.fits),
                                 name_things='triple points',
                                 perc_interv=5)
        for fit in self.fits:
            try:
                fit.detect_triple_points(use_x_minima=use_x_minima)
            except Exception:
                pass
            if verbose:
                pg.print_progress()
        if smooth is not None:
            self.smooth_triple_points(tos='gaussian', size=smooth)

    def display(self, displ_tp=True, displ_ca=True):
        super().display(displ_tp=displ_tp, displ_ca=displ_ca)
        displs = []
        # Display spline fitting
        x1s = []
        y1s = []
        x2s = []
        y2s = []
        for fit in self.fits:
            if fit is None:
                continue
            t = np.linspace(0, 1, 1000)
            x1 = fit.fits[0][0](t)
            y1 = fit.fits[0][1](t)
            x2 = fit.fits[1][0](t)
            y2 = fit.fits[1][1](t)
            if np.any(np.isnan(x2)):
                filt = np.logical_not(np.isnan(x2))
                x2 = x2[filt]
                y2 = y2[filt]
            if np.any(np.isnan(x1)):
                filt = np.logical_not(np.isnan(x1))
                x1 = x1[filt]
                y1 = y1[filt]
            x1s.append(x1)
            x2s.append(x2)
            y1s.append(y1)
            y2s.append(y2)
        x1s = np.asarray(x1s)
        x2s = np.asarray(x2s)
        y1s = np.asarray(y1s)
        y2s = np.asarray(y2s)
        if x1s.shape[1] != 0:
            db1 = pplt.Displayer(x1s, y1s, color=self[0].colors[1])
            displs.append(db1)
        if x2s.shape[1] != 0:
            db2 = pplt.Displayer(x2s, y2s, color=self[0].colors[1])
            displs.append(db2)
        # Add button manager
        if len(displs) != 0:
            bm = pplt.ButtonManager(displs)
            return bm


class TemporalCircleFits(TemporalFits):
    def display(self, displ_tp=True, displ_ca=True):
        super().display(displ_tp=displ_tp, displ_ca=displ_ca)
        displs = []
        # Display circle
        thetas = np.linspace(0, np.pi*2, 100)
        xs = []
        ys = []
        xps = []
        yps = []
        # Loop on times
        for fit in self.fits:
            if fit.fits is None:
                for l in [xs, ys]:
                    l.append(np.nan*thetas)
                for l in [xps, yps]:
                    l.append([np.nan])
                continue
            # plot the circle !
            (xc, yc), R = fit.fits
            xs.append(xc + R*np.cos(thetas))
            ys.append(yc + R*np.sin(thetas))
            xps.append([xc])
            yps.append([yc])
        if not np.all(np.isnan(ys)):
            db = pplt.Displayer(xs, ys, kind='plot',
                                color=self[0].colors[5])
            displs.append(db)
        if not np.all(np.isnan(yps)):
            dbp = pplt.Displayer(xps, yps, kind='plot',
                                 marker='o',
                                 color=self[0].colors[5])
            displs.append(dbp)
        # Add button manager
        if len(displs) != 0:
            bm = pplt.ButtonManager(displs)
            return bm


class TemporalEllipseFits(TemporalFits):
    def display(self, displ_tp=True, displ_ca=True):
        super().display(displ_tp=displ_tp, displ_ca=displ_ca)
        displs = []
        # Display ellipse fits
        res = 100
        xs = []
        ys = []
        xps = []
        yps = []
        # Loop on times
        for fit in self.fits:
            if fit.fits is None:
                for l in [xs, ys]:
                    l.append([np.nan]*res)
                for l in [xps, yps]:
                    l.append([np.nan])
                continue
            # plot the circle !
            (xc, yc), R1, R2, theta = fit.fits
            elxs, elys = hlp.get_ellipse_points(xc, yc, R1, R2, theta)
            xs.append(elxs)
            ys.append(elys)
            xps.append([xc])
            yps.append([yc])
        if not np.all(np.isnan(ys)):
            db = pplt.Displayer(xs, ys, kind='plot',
                                color=self[0].colors[5])
            displs.append(db)
        if not np.all(np.isnan(yps)):
            dbp = pplt.Displayer(xps, yps, kind='plot',
                                 marker='o',
                                 color=self[0].colors[5])
            displs.append(dbp)
        # Add button manager
        if len(displs) != 0:
            bm = pplt.ButtonManager(displs)
            return bm


class TemporalEllipsesFits(TemporalFits):
    def display(self, displ_tp=True, displ_ca=True):
        super().display(displ_tp=displ_tp, displ_ca=displ_ca)
        displs = []
        # Display ellipse fits
        res = 100
        xs, ys = [], []
        xps, yps = [], []
        xs2, ys2 = [], []
        xps2, yps2 = [], []
        # Loop on times
        for fit in self.fits:
            if fit.fits is None:
                for l in [xs, ys]:
                    l.append([np.nan]*res)
                for l in [xps, yps]:
                    l.append([np.nan])
                continue
            # First ellipse
            (xc, yc), R1, R2, theta = fit.fits[0]
            elxs, elys = hlp.get_ellipse_points(xc, yc, R1, R2, theta)
            filt = elxs > xc
            elxs[filt] = np.nan
            elys[filt] = np.nan
            xs.append(elxs)
            ys.append(elys)
            xps.append([xc])
            yps.append([yc])
            # Second ellipse
            (xc, yc), R1, R2, theta = fit.fits[1]
            elxs, elys = hlp.get_ellipse_points(xc, yc, R1, R2, theta)
            filt = elxs < xc
            elxs[filt] = np.nan
            elys[filt] = np.nan
            xs2.append(elxs)
            ys2.append(elys)
            xps2.append([xc])
            yps2.append([yc])
        # First ellipse
        if not np.all(np.isnan(ys)):
            db = pplt.Displayer(xs, ys, kind='plot',
                                color=self[0].colors[5])
            displs.append(db)
        if not np.all(np.isnan(yps)):
            dbp = pplt.Displayer(xps, yps, kind='plot',
                                 marker='o',
                                 color=self[0].colors[5])
            displs.append(dbp)
        # Second ellipse
        if not np.all(np.isnan(ys2)):
            db = pplt.Displayer(xs2, ys2, kind='plot',
                                color=self[0].colors[5])
            displs.append(db)
        if not np.all(np.isnan(yps2)):
            dbp = pplt.Displayer(xps2, yps2, kind='plot',
                                 marker='o',
                                 color=self[0].colors[5])
            displs.append(dbp)
        # Add button manager
        if len(displs) != 0:
            bm = pplt.ButtonManager(displs)
            return bm


class TemporalCirclesFits(TemporalFits):
    def display(self, displ_tp=True, displ_ca=True):
        super().display(displ_tp=displ_tp, displ_ca=displ_ca)
        displs = []
        # Display circles
        thetas = np.linspace(0, np.pi*2, 100)
        # loop on different circles
        for i in range(3):
            xs = []
            ys = []
            xps = []
            yps = []
            # Loop on times
            for fit in self.fits:
                if fit.fits is None:
                    for l in [xs, ys]:
                        l.append(np.nan*thetas)
                    for l in [xps, yps]:
                        l.append([np.nan])
                    continue
                if i >= len(fit.fits):
                    for l in [xs, ys]:
                        l.append(np.nan*thetas)
                    for l in [xps, yps]:
                        l.append([np.nan])
                    continue
                if fit.fits[i] is None:
                    for l in [xs, ys]:
                        l.append(np.nan*thetas)
                    for l in [xps, yps]:
                        l.append([np.nan])
                    continue
                # plot the circle !
                (xc, yc), R = fit.fits[i]
                xs.append(xc + R*np.cos(thetas))
                ys.append(yc + R*np.sin(thetas))
                xps.append([xc])
                yps.append([yc])
            if not np.all(np.isnan(ys)):
                db = pplt.Displayer(xs, ys, kind='plot',
                                    color=self[0].colors[5])
                displs.append(db)
            if not np.all(np.isnan(yps)):
                dbp = pplt.Displayer(xps, yps, kind='plot',
                                     marker='o',
                                     color=self[0].colors[5])
                displs.append(dbp)
        # Add button manager
        if len(displs) != 0:
            bm = pplt.ButtonManager(displs)
            return bm



# -*- coding: utf-8 -*-
#!/bin/env python3

# Copyright (C) 2018 Gaby Launay

# Author: Gaby Launay  <gaby.launay@tutanota.com>
# URL: https://github.com/gabylaunay/pyDSA_core

# This file is part of pyDSA_core

# pyDSA_core is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

import cv2
import os
from glob import glob
import numpy as np
from . import temporalimages as tis
from . import image
from IMTreatment.utils import ProgressCounter, make_unit
import IMTreatment.file_operation as imtio
import warnings
import scipy.optimize as spopt

"""  """

__author__ = "Gaby Launay"
__copyright__ = "Gaby Launay 2017"
__credits__ = ""
__email__ = "gaby.launay@tutanota.com"
__status__ = "Development"


def import_from_image(path, dx=1, dy=1, unit_x="", unit_y="",
                      cache_infos=True, dtype=np.uint8):
    """
    Import an image into an Image object.

    Parameters
    ==========
    path: string
        Path to the image file.
    dx, dy: numbers
        Real distance between two pixels.
    unit_x, unit_y: strings
        Unities of dx and dy.

    Returns
    =======
    img: Image object
        Image
    """
    imtio.check_path(path)
    data = cv2.imread(path, cv2.IMREAD_GRAYSCALE)
    if data is None:
        raise IOError(f"{path} is not a valid image.")
    data = data.transpose()
    data = data[:, ::-1]
    axe_x = np.arange(0, data.shape[0]*dx, dx)
    axe_y = np.arange(0, data.shape[1]*dy, dy)
    img = image.Image(filepath=path, cache_infos=cache_infos)
    img.import_from_arrays(axe_x, axe_y, data,
                           unit_x=unit_x, unit_y=unit_y,
                           dtype=dtype)
    if cache_infos:
        # Try to import infos if the infofile exist
        if os.path.isfile(img.infofile_path):
            img._import_infos()
        # cache axis
        img._dump_infos()
    return img


def import_from_images(path, dx=1, dy=1, dt=1, unit_x="", unit_y="",
                       unit_times="", dtype=np.uint8,
                       iteration_hook=None, verbose=False):
    """
    Import a set of images into an TemporalImages object.

    Parameters
    ==========
    path: string
        Path to the images directory.
    dx, dy: numbers
        Real distance between two pixels.
    dt: numbers
        Time interval between two images.
    unit_x, unit_y, unit_times: strings
        Unities of dx, dy and time.

    Returns
    =======
    imgs: TemporalImages object
        Images
    """
    dirname = os.path.dirname(path)
    imtio.check_path(dirname)
    paths = glob(path)
    paths = sorted(paths)
    ims = tis.TemporalImages(dirname, cache_infos=True)
    t = 0
    # verbose
    if verbose:
        pg = ProgressCounter("Importing images", len(paths),
                             end_mess="Done", name_things="images")
    for i, path in enumerate(paths):
        im = import_from_image(path=path, dx=dx, dy=dy, unit_y=unit_y,
                               unit_x=unit_x, cache_infos=False,
                               dtype=dtype)
        ims.add_field(im, time=t, unit_times=unit_times)
        t += dt
        if verbose:
            pg.print_progress()
        if iteration_hook is not None:
            iteration_hook(i, len(paths))
    # load saved things if necessary
    ims._import_infos()
    return ims


def import_from_video(path, dx=1, dy=1, dt=None,
                      unit_x="", unit_y="", unit_t="",
                      frame_inds=None, frame_range=None,
                      incr=1, nmb_frame_to_import=None,
                      intervx=None, intervy=None,
                      cache_infos=True, dtype=np.uint8,
                      iteration_hook=None,
                      verbose=False):
    """
    Import a images from a video file.

    Parameters
    ==========
    path: string
        Path to the video file.
    dx, dy: numbers
        Real distance between two pixels.
    dt: number, optional
        Time interval between two frames.
        If not specified, the time interval is taken from the video metadata
    unit_x, unit_y, unit_y: strings
        Unities of dx, dy and dt.
    frame_range: 2x1 array of integers
        Range of frame to import (default to all).
    frame_inds: array of integers
        Indices of frames to import (default to all).
    incr: integer
        Number of frame to import.
        (ex: with a value of 2, only 1/2 frames will be imported).
    nmb_frame_to_import: integer
        Number of evenly distributed frames to import
        Will overwrite any values of 'incr'
    cache_infos: boolean
        If True (default) an infofile is created alongside the video file
        To keep track of scaling preoperties and baseline.
    intervx, intervy: 2x1 list of numbers
        Cropping dimensions applied on each frames.
    dtype: type
        Numerical type used for the stored data.
        Should be a type supported by numpy arrays.
        Default to 8 bit unsigned integers (np.uint8) to optimize memory usage.

    Returns
    =======
    imgs: TemporalImages object
        Images
    """
    # check
    if frame_inds is not None and frame_range is not None:
        raise Exception()
    if frame_inds is not None and incr != 1:
        raise Exception("You cannot specify both the 'frame_inds' and an "
                        "'incr' value")
    if frame_inds is not None and nmb_frame_to_import is not None:
        raise Exception("You cannot specify both the 'frame_inds' and an "
                        "'nmb_frame_to_import' value")
    # Check if file exist
    imtio.check_path(path)
    # convert units (upfront for optimization purpose)
    unit_t = make_unit(unit_t)
    unit_x = make_unit(unit_x)
    unit_y = make_unit(unit_y)
    # open video
    vid = cv2.VideoCapture()
    vid.open(path)
    ti = tis.TemporalImages(filepath=path)
    i = 0
    # Get indices to import
    max_frame = int(vid.get(cv2.CAP_PROP_FRAME_COUNT))
    if max_frame == 0:
        raise ImportError("Couldn't get the number of frame from the video file")
    if frame_inds is not None:
        if frame_inds[-1] > max_frame:
            raise Exception()
    else:
        if frame_range is None:
            frame_max = max_frame - 1
            frame_min = 0
        else:
            if frame_range[0] < 0:
                frame_min = 0
            else:
                frame_min = frame_range[0]
            if frame_range[1] > max_frame:
                frame_max = max_frame
            else:
                frame_max = frame_range[1]
        if nmb_frame_to_import is not None:
            incr = int((frame_max - frame_min)/nmb_frame_to_import)
            if incr == 0:
                incr = 1
        frame_inds = np.arange(frame_min, frame_max, incr, dtype=int)
    # Get fps
    if dt is None:
        fps = float(vid.get(cv2.CAP_PROP_FPS))
        if fps == 0:
            fps = 1
        dt = 1/fps
    # logs
    if verbose:
        nmb_frames = len(frame_inds)
        if nmb_frames <= 1:
            raise Exception("No frames selected")
        pg = ProgressCounter(init_mess="Decoding video",
                             nmb_max=nmb_frames,
                             name_things='frames',
                             perc_interv=5)
    t = 0
    for i in np.arange(0, frame_inds[-1] + 1, 1):
        if i not in frame_inds:
            t += dt
            continue
        vid.set(cv2.CAP_PROP_POS_FRAMES, i)
        success, im = vid.read()
        if not success:
            if frame_inds[1] != np.inf:
                warnings.warn(f"Can't decode frame number {i}")
            break
        im = cv2.cvtColor(im, cv2.COLOR_RGB2GRAY)
        # im = np.mean(im, axis=2)
        im = im.transpose()[:, ::-1]
        axe_x = np.arange(0, im.shape[0]*dx - 0.1*dx, dx)
        axe_y = np.arange(0, im.shape[1]*dy - 0.1*dy, dy)
        sf = image.Image()
        sf.import_from_arrays(axe_x, axe_y, im, mask=False,
                              unit_x=unit_x, unit_y=unit_y,
                              dontchecknans=True,
                              dontcheckunits=True,
                              dtype=dtype)
        sf.crop(intervx=intervx, intervy=intervy, inplace=True)
        ti.add_field(sf, time=t, unit_times=unit_t, copy=False)
        t += dt
        if verbose:
            pg.print_progress()
        if iteration_hook is not None:
            iteration_hook(i, frame_inds[-1] + 1)

    if cache_infos:
        # Try to import infos if the infofile exist
        if os.path.isfile(ti.infofile_path):
            ti._import_infos()
        # cache axis
        ti._dump_infos()
    #
    if verbose and frame_inds[1] == np.inf:
        pg._print_end()
    if len(ti.fields) == 0:
        raise Exception("Something goes wrong during video import, "
                        "the video does not contain any image...")
    return ti


def circle_from_three_points(pt1, pt2, pt3):
    """
    Give the center and radius of a circle from three points
    """
    x1, y1 = pt1
    x2, y2 = pt2
    x3, y3 = pt3
    # ensure no division by zero
    if (x2 - x1) == 0:
        ma = (y3 - y1)/(x3 - x1)
        mb = (y3 - y2)/(x3 - x2)
    elif (x3 - x2) == 0:
        ma = (y2 - y1)/(x2 - x1)
        mb = (y3 - y1)/(x3 - x1)
    else:
        ma = (y2 - y1)/(x2 - x1)
        mb = (y3 - y2)/(x3 - x2)
    if (mb - ma) == 0:
        x = np.mean([x1, x2, x3])
    else:
        x = (ma*mb*(y1 - y3) + mb*(x1 + x2) - ma*(x2 + x3))/(2*(mb - ma))
    if ma == 0:
        y = np.mean([y1, y2, y3])
    else:
        y = -(1/ma)*(x - (x1 + x2)/2) + (y1 + y2)/2
    R = ((y1 - y)**2 + (x1 - x)**2)**.5
    return (x, y), R


def fit_circle(xs, ys, baseline=None, tangent_circ=None, sigma_max=None,
               soft_constr=False):
    """
    Fit a circle to the given points.

    Parameters
    ==========
    xs, ys: arrays
        Coordinates of the points to fit.
    baseline: Baseline object
        If specified, the fitting will try to not pass through the baseline.
    tangent_circ: ((x, y), R) tuple
        Circle to be tangent to the fitting
    sigma_max: number
        If specified, points too far from the fit are iteratively removed
        until they all fall in the range R +- sigma*R
    soft_constr: boolean
        If True, constraints are fitted instead of being imposed.
    """
    # Init some params
    if baseline is not None:
        basefun = baseline.get_baseline_fun()
    if tangent_circ is not None:
        (xd, yd), Rd = tangent_circ
        if xd > np.mean(xs):
            side = -1
        else:
            side = +1
    # # Compute weight from points density
    # # (to not give too much power to high density regions)
    # # DOES NOT WORK BECAUSE THE POINTS ARE NOT SORTED !
    # if np.all(ys[1::] - ys[:-1] > 0):
    #     indsort = np.argsort(ys)
    #     ys = ys[indsort]
    #     xs = xs[indsort]
    # weights = []
    # for i in range(len(ys)):
    #     if i == 0:
    #         weights.append(((ys[i] - ys[i+1])**2 + (xs[i] - xs[i+1])**2)**.5)
    #     elif i == len(ys) - 1:
    #         weights.append(((ys[i] - ys[i-1])**2 + (xs[i] - xs[i-1])**2)**.5)
    #     else:
    #         d1 = ((ys[i] - ys[i+1])**2 + (xs[i] - xs[i+1])**2)**.5
    #         d2 = ((ys[i] - ys[i-1])**2 + (xs[i] - xs[i-1])**2)**.5
    #         weights.append((d1 + d2)/2)
    # weights = np.asarray(weights)
    # weights /= np.sum(weights)/len(weights)
    # # TEMP
    # weights = np.ones(len(weights))
    # # TEMP - End

    def calc_R(x, y, xc, yc):
        """
        Calculate the distance of each data points from the center (xc, yc)
        """
        return np.sqrt((x - xc)**2 + (y - yc)**2)

    def f_circle_soft_constr(args, x, y):
        """
        Calculate the algebraic distance between the 2D points and the mean
        circle centered at c=(xc, yc)
        """
        xc, yc = args
        Ri = calc_R(x, y, xc, yc)
        residu = (Ri - Ri.mean())
        if baseline is not None:
            yb = basefun(xc)
            residu += (Ri.mean() - (yc - yb))
        if tangent_circ is not None:
            (xo, yo), Ro = tangent_circ
            dist_circ = ((xo - xc)**2 + (yo - yc)**2)**.5
            residu += (dist_circ - (Ro + Ri.mean()))
        return residu

    def f_circle_constr(args, x, y):
        """
        Calculate the algebraic distance between the 2D points and the mean
        circle centered at c=(xc, yc)
        """
        R = abs(args[0])
        (xd, yd), Rd = tangent_circ
        # first y guess
        xc, yc = center_from_R(R)
        # return residu
        Ri = calc_R(x, y, xc, yc)
        residu = (Ri - R)
        return residu

    def f_circle(args, x, y):
        """
        Calculate the algebraic distance between the 2D points and the mean
        circle centered at c=(xc, yc)
        """
        xc, yc = args
        Ri = calc_R(x, y, xc, yc)
        residu = (Ri - Ri.mean())
        return residu

    def center_from_R(R):
        """
        Return the position of the fitting center for a
        constrained circle of diameter R
        """
        (xd, yd), Rd = tangent_circ
        # first y guess
        yc = basefun(xd - Rd - R) + R
        # first x guess
        xc = xd + side*((R + Rd)**2 - (yc - yd)**2)**.5
        # corrections
        old_yc = np.inf
        while abs(old_yc - yc)/R > 1e-6:
            old_yc = yc
            yc = basefun(xc) + R
            xc = xd + side*((R + Rd)**2 - (yc - yd)**2)**.5
        return xc, yc

    # First guess from three points
    (xc, yc), R = circle_from_three_points([xs[0], ys[0]],
                                           [xs[int(len(xs)/2)],
                                            ys[int(len(xs)/2)]],
                                           [xs[-1], ys[-1]])
    # Choose adequate fitting function
    if baseline is None and tangent_circ is None:
        fit_function = f_circle
        init_guess = (xc, yc)
    elif baseline is not None and tangent_circ is not None:
        # checking side
        if soft_constr:
            fit_function = f_circle_soft_constr
            init_guess = (xc, yc)
        else:
            fit_function = f_circle_constr
            init_guess = (R,)
    else:
        raise Exception("Not implemented yet")
    # Iterative fit (removing marginal values each time)
    tmp_xs = xs.copy()
    tmp_ys = ys.copy()
    if sigma_max is not None:
        while True:
            args, ier = spopt.leastsq(
                fit_function, init_guess,
                args=(tmp_xs, tmp_ys))
            if len(args) == 1:
                R = abs(args[0])
                center = center_from_R(R)
                Rs = calc_R(tmp_xs, tmp_ys, *center)
            else:
                center = args
                Rs = calc_R(tmp_xs, tmp_ys, *center)
                R = np.mean(Rs)
            R_std = np.std(Rs)
            if R_std < R*sigma_max or len(tmp_xs) <= 10:
                break
            filt = np.logical_and(Rs < R + R_std*2,
                                  Rs > R - R_std*2)
            if np.all(filt):
                filt[np.argmax(abs(Rs - R))] = False
            tmp_xs = tmp_xs[filt]
            tmp_ys = tmp_ys[filt]
    # Classical fit
    else:
        args, ier = spopt.leastsq(
            fit_function, init_guess,
            args=(tmp_xs, tmp_ys))
        if len(args) == 1:
            R = abs(args[0])
            center = center_from_R(R)
        else:
            center = args
            R = np.mean(calc_R(tmp_xs, tmp_ys, *center))
    # return
    return center, R


def fit_ellipse(xs, ys):
    """
    Fit an Ellispe to the given set of points.

    Parameters
    ----------
    xs, ys: arrays
        Coordinates of the points to fit.

    Returns
    -------
    center: 2x1 array of numbers
        Position of the ellipse center.
    R1, R2: numbers
        Main and secondary ellipse radii.
    theta: angle
        Angle of the ellispe main axis (in radian).
    """

    # Taken from: http://nicky.vanforeest.com/misc/fitEllipse/fitEllipse.html
    # Get a
    xs = np.asarray(xs)[:, np.newaxis]
    ys = np.asarray(ys)[:, np.newaxis]
    D = np.hstack((xs*xs, xs*ys, ys*ys, xs, ys, np.ones_like(xs)))
    S = np.dot(D.T, D)
    C = np.zeros([6, 6])
    C[0, 2] = C[2, 0] = 2
    C[1, 1] = -1
    E, V = np.linalg.eig(np.dot(np.linalg.inv(S), C))
    n = np.argmax(E)
    lambd = V[:, n]
    b, c, d, f, g, a = (lambd[1]/2, lambd[2], lambd[3]/2,
                        lambd[4]/2, lambd[5], lambd[0])
    # Get ellipse center
    num = b*b - a*c
    x0 = (c*d - b*f) / num
    y0 = (a*f - b*d) / num
    xc = np.array([x0, y0])
    # Get ellispe rotation
    theta = 0.5*np.arctan(2*b / (a - c))
    # Get ellipse radii
    up = 2*(a*f*f + c*d*d + g*b*b - 2*b*d*f - a*c*g)
    down1 = (b*b - a*c)*((c - a)*np.sqrt(1 + 4*b*b / (
        (a - c)*(a - c))) - (c + a))
    down2 = (b*b - a*c)*((a - c)*np.sqrt(1 + 4*b*b / (
        (a - c)*(a - c))) - (c + a))
    R1 = np.sqrt(up / down1)
    R2 = np.sqrt(up / down2)
    # Return
    return xc, R1, R2, theta


def get_ellipse_points(xc, yc, R1, R2, theta, res=100):
    """
    Return points on the coresponding ellipse.
    (For plotting purposes)
    """
    thetas = np.linspace(0, np.pi*2, res)
    xs = xc + R1*np.cos(thetas)
    ys = yc + R2*np.sin(thetas)
    rot_xs = xc + np.cos(theta)*(xs - xc) - np.sin(theta)*(ys - yc)
    rot_ys = yc + np.cos(theta)*(ys - yc) + np.sin(theta)*(xs - xc)
    return rot_xs, rot_ys



# -*- coding: utf-8 -*-
#!/bin/env python3

# Copyright (C) 2003-2007 Gaby Launay

# Author: Gaby Launay  <gaby.launay@tutanota.com>
# URL: https://github.com/gabylaunay/pyDSA_core


# This file is part of pyDSA_core

# pyDSA_core is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

from setuptools import setup, find_packages
from os import path

here = path.abspath(path.dirname(__file__))

with open(path.join(here, "README.md"), "r") as fh:
    long_description = fh.read()

setup(
    name='pyDSA_core',
    version='1.1.1',
    description='Python Drop Shape Analyzer',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://framagit.org/gabylaunay/pyDSA_core',
    author='Gaby Launay',
    author_email='gaby.launay@protonmail.com',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Science/Research',
        'Topic :: Scientific/Engineering',
        'Programming Language :: Python :: 3.5',
        'Natural Language :: English',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Operating System :: POSIX :: Linux',
    ],
    keywords='DSA drop shape contact angle hysteresis',
    packages=find_packages(exclude=['contrib', 'docs', 'tests', 'samples']),
    install_requires=['numpy', 'matplotlib', 'opencv-python', 'scipy',
                      'imageio', 'scikit-image', 'IMTreatment==1.1.1'],
    extras_require={},
    setup_requires=['pytest-runner'],
    tests_require=['pytest'],
    zip_safe=False,
)



# -*- coding: utf-8 -*-
#!/bin/env python3

# Copyright (C) 2018 Gaby Launay

# Author: Gaby Launay  <gaby.launay@tutanota.com>
# URL: https://github.com/gabylaunay/pyDSA_core

# This file is part of pyDSA_core

# pyDSA_core is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
from .image import Image
from .temporalimages import TemporalImages
from .dropedges import DropEdges
from .dropfit import DropFit, DropCircleFit, DropCirclesFit, DropEllipseFit, \
    DropSplineFit
from .helpers import import_from_video, import_from_image, import_from_images

name = "pyDSA_core"
